open Library
open Machine
open Types

(* Type of name-name constraints used within the solver.
 * Abstractions are pulled out to the front, with the innermost
 * binders at the heads of the lists. *)
type constr = EQ of (int * int list * mvalue) * (int * int list * mvalue)
            | FR of int * (int * int list * mvalue)

(* Make equality and freshness constraints. *)
let mkEq mv mv' = EQ((0,[],mv),(0,[],mv'))
let mkFr x mv = FR(x,(0,[],mv))

(* Representation of constraint problems. *)
type prob = constr list

(* The true constraint problem *)
let trueProb = []

(* Produce a string representation of a constraint problem. *)
let string_of_constr vd c =
  let soevar i =
    ("(" ^ (string_of_evar i) ^ ":" ^ (soType(Vardata.getType vd i)) ^ ")")
  in
  let soAbs (xs,mv) =
    (string_of_enclist2 "<" ">" soevar "><" xs) ^ (soMValueGeneral soevar mv)
  in
  match c with
    EQ((nx,xs,v),(ny,ys,v')) ->
      (soAbs(xs,v)) ^ " = " ^ (soAbs(ys,v'))
  | FR(x,(ny,ys,v)) ->
      (soevar x) ^ " # " ^ (soAbs(ys,v))

(* Produce a string representation of a constraint problem.
 * This version may walk the evars in the constraints. *)
let string_of_constr2 printevar vd c =
  let soevar i =
    ("(" ^ (printevar i) ^ ":" ^ (soType(Vardata.getType vd i)) ^ ")")
  in
  let soAbs (xs,mv) =
    (string_of_enclist2 "<" ">" soevar "><" xs) ^ (soMValueGeneral soevar mv)
  in
  match c with
    EQ((nx,xs,v),(ny,ys,v')) ->
      (soAbs(xs,v)) ^ " = " ^ (soAbs(ys,v'))
  | FR(x,(ny,ys,v)) ->
      (soevar x) ^ " # " ^ (soAbs(ys,v))

(* Produce a string representation of a problem,
 * for debugging purposes... *)
let string_of_prob printevar vd cs =
  string_of_enclist "{" "}" (string_of_constr2 printevar vd) ",\n " cs
