(** Datatypes for location data, used during parsing to (hopefully) give accurate error reporting. *)

open Lexing

(** Type identifying a location *)
type t = { line : int; character : int }

(** Type identifying a span across 2 locations *)
type span = { start : t; finish : t }

(** A dummy location and span, useful in some cases *)
val dummy_loc : t
val dummy_span : span

(** Create a location from a lexing position *)
val loc_from_pos : Lexing.position -> t

(** Create a span from 2 lexing positions *)
val span_from_poss : Lexing.position -> Lexing.position -> span

(** Create a span from 2 locations *)
val span_from_locs : t -> t -> span

(** Merge two spans *)
val mergeSpans : span -> span -> span

(** Highlight where a span would appear in the console *)
val highlight : span -> unit

(** to_string functions *)
val to_string : t -> string
val span_to_string : span -> string

(** Imperative state storing the locations of the last lexeme seen.
    Helps to give meaningful syntax error messages, as Ocamlyacc is less than helpful in that respect! *)
val curr_lexeme_start : t ref
val curr_lexeme_finish : t ref
val curr_lexeme_reset : unit -> unit
