open Types
open Machine

(* Store information about variables - their type and (maybe) a substitution. *)
type t

(* Empty vardata structure. *)
val empty : t

(* Add the next evar along, with the appropriate type. *)
val addEvar : t -> ty -> (int * t)

(* Add the next evar along, with the appropriate type and a substitution. *)
val addEvarWithSubst : t -> ty -> mvalue -> (int * t)

(* Add a substitution to an existing evar. *)
val addSubst : t -> int -> mvalue -> t

(* Get the substitution from an existing evar, if any. *)
val getSubst : t -> int -> (mvalue option)

(* Get the type from an existing evar. *)
val getType : t -> int -> ty

(* Get the last assigned evar (returns -1 if none assigned yet). *)
val lastEvar : t -> int

(* Produce a string representation of the SUBSTITUTION part.
 * Functions must be supplied for printing out evars and mvalues. *)
val string_of_subst : (int -> string) -> (mvalue -> string) -> t -> string

(* Get the list of all mappings. *)
val get_mappings : t -> (int * mvalue) list
